/***************************************************************************
                          kconnections.cpp  -  description
                             -------------------
    begin                : Втр Дек  4 20:35:42 UTC 2007
    copyright            : (C) 2007 by Krasu
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qtimer.h>
#include <qtooltip.h>
#include <qstringlist.h>

#include <kglobal.h>
#include <kprocess.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kdebug.h>
#include <kwin.h>
#include <khelpmenu.h>
#include <kapplication.h>
#include <kaction.h>

#include "kconnections.h"
#include "kwmonitor.h"
#include "kwentry.h"
#include "kwlabel.h"
#include "kwoptions.h"
#include "kwconfig.h"
#include "kwignore.h"

KWDynamicTip::KWDynamicTip(QWidget *parent): QToolTip(parent)
{}

void KWDynamicTip::maybeTip(const QPoint &)
{
    KConnections *kc = (KConnections *)parentWidget();
    QRect r(kc->rect());

    if(!r.isValid())
        return;

    if(kc->timeToo())
        kc->listChanged(); // also calc new 'text'

    tip(r, text);
}

KConnections::KConnections(QWidget *parent, const char *name) : KSystemTray(parent, name)
{
    proc = new KProcess;
    (*proc) << NETSTAT << "--inet" << "-e" << "-n" << "-p";
    connect(proc, SIGNAL(receivedStdout(KProcess *, char *, int)), this, SLOT(slotReceivedStdout(KProcess *, char *, int)));
    connect(proc, SIGNAL(processExited(KProcess *)), this, SLOT(slotExited()));

    kconf = new KWConfig;

    if(!kconf->hasGroup("Ignore"))
    {
        kconf->setGroup("Ignore");
        kconf->writeEntry("0", "1,-,www,-,,,,,");
        kconf->writeEntry("1", "1,-,netbios-ssn,-,,,,,");
        kconf->writeEntry("2", "1,-,microsoft-ds,-,,,,,");
    }

    if(!kconf->hasGroup("Accept"))
    {
        kconf->setGroup("Accept");
        kconf->writeEntry("0", "1,-,ssh,-,,,,,");
    }

    ign = new KWIgnore;
    label = new KWLabel;
    entry = new KWEntry;

    kwm = new KWMonitor;

    connect(this, SIGNAL(added(const Entry &)), kwm, SLOT(slotAdded(const Entry &)));
    connect(this, SIGNAL(deleted(const Entry &)), kwm, SLOT(slotDeleted(const Entry &)));

    KPopupMenu *p = contextMenu();
    p->insertItem(i18n("Help"), (new KHelpMenu(this, kapp->aboutData()))->menu());
    p->insertItem(KGlobal::iconLoader()->loadIcon("configure", KIcon::Desktop, KIcon::SizeSmall), i18n("Configure..."), this, SLOT(slotOptions()));
    p->insertSeparator();
    KRadioAction *r1 = new KRadioAction(i18n("Accept all connections"), "", 0, this, SLOT(slotAcceptAll()), actionCollection(), "kconnections_type0");
    KRadioAction *r2 = new KRadioAction(i18n("Ignore all connections"), "", 0, this, SLOT(slotIgnoreAll()), actionCollection(), "kconnections_type1");
    KRadioAction *r3 = new KRadioAction(i18n("Use black list"), "", 0, this, SLOT(slotBlack()), actionCollection(), "kconnections_type2");
    KRadioAction *r4 = new KRadioAction(i18n("Use white list"), "", 0, this, SLOT(slotWhite()), actionCollection(), "kconnections_type3");
    r1->plug(p);
    r2->plug(p);
    r3->plug(p);
    r4->plug(p);
    p->insertSeparator();

    const QString conn = QString::fromLatin1("connection_type");
    r1->setExclusiveGroup(conn);
    r2->setExclusiveGroup(conn);
    r3->setExclusiveGroup(conn);
    r4->setExclusiveGroup(conn);

    connect(entry, SIGNAL(added(const Entry &)), this, SLOT(slotAdded(const Entry &)));
    connect(entry, SIGNAL(added(const Entry &)), this, SIGNAL(added(const Entry &)));

    setPixmap(KGlobal::iconLoader()->loadIcon("kconnections", KIcon::Desktop, KIcon::SizeSmall));

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(startMonHack()));

    connect(this, SIGNAL(quitSelected()), this, SLOT(slotQuitRequest()));

    rereadSettings();

    timer->start(10, true);
}

KConnections::~KConnections()
{
    delete kwm;
    delete entry;
    delete label;
    delete ign;
    delete kconf;
    delete proc;
}

void KConnections::startMonHack()
{
    updateGeometries();

    disconnect(timer, SIGNAL(timeout()), 0, 0);
    connect(timer, SIGNAL(timeout()), this, SLOT(startMon()));

    timer->start(0, true);
}

void KConnections::startMon()
{
    buffer = "";

    proc->start(KProcess::NotifyOnExit, KProcess::Stdout);
}

void KConnections::slotExited()
{
    QStringList list = QStringList::split(QChar('\n'), buffer);

    entry->unmark();

    // ignore netstat's header
    if(list.size() > 2)
    {
        list.pop_front();
        list.pop_front();

        processList(list);
    }

    KWEntry::iterator itEnd = entry->end();

    for(KWEntry::iterator it = entry->begin();it != itEnd;)
    {
        if(!(*it).mark)
        {
            emit deleted(*it);

           if(messages && messages_discon)
                label->addText((*it), KWLabel::Deleted);

            it = entry->erase(it); // also ++it
            itEnd = entry->end();

            listChanged();
        }
        else
            ++it;
    }

    timer->start(delay, true);
}

void KConnections::slotReceivedStdout(KProcess *, char *buf, int buflen)
{
    char ba[buflen+1];
    memcpy(ba, buf, buflen);
    ba[buflen] = '\0';

    buffer.append(ba);
}

void KConnections::processList(const QStringList &list)
{
    QStringList::const_iterator itEnd = list.end();

    for(QStringList::const_iterator it = list.begin();it != itEnd;++it)
    {
        entry->splitAndAdd(*it); // also can emit added()
    }
}

void KConnections::slotAdded(const Entry &e)
{
    if(messages && messages_con)
        label->addText(e, KWLabel::Added);

    listChanged();
}

void KConnections::listChanged()
{
    if(!entry->count())
    {
        tip->setText(i18n("No connections"));
        return;
    }

    QString result = i18n("1 connection", "%n connections", entry->count())
                     + "<table cellspacing=0 cellpadding=0>\n";

    KWEntry::iterator itEnd = entry->end();
    int cnt = 0;

    for(KWEntry::iterator it = entry->begin();it != itEnd;++it)
    {
        if(((*it).incoming && !label->incomingSett()) || (!(*it).incoming && !label->outgoingSett()))
            continue;

        cnt++;

        result += "<tr>";
        result += (*it).tableEntry();
        result += "</tr>";
    }

    if(cnt)
    {
        result += "</table>";
        tip->setText(result);
    }
    else
        tip->setText(i18n("No connections"));
}

void KConnections::slotOptions()
{
    KWOptions kwo;

    if(kwo.start() == QDialog::Accepted)
    {
        label->rereadSettings();
        ign->reread();
        rereadSettings();
        listChanged();
    }
}

void KConnections::slotQuitRequest()
{
    kconf->sync();
}

void KConnections::mousePressEvent(QMouseEvent *e)
{
    if(e->button() == Qt::LeftButton)
        kwm->show();
    else
        KSystemTray::mousePressEvent(e);
}

void KConnections::rereadSettings()
{
    kconf->setGroup("General");

    delay = kconf->readNumEntry("rescan_delay", 1000);

    if(delay < 0 || delay > 30000)
        delay = 1000;

    inctime = kconf->readEntry("message_format", "<td><b>%p</b></td> <td><b>%s</b></td> <td><b>%us</b></td> <td>%l:%lp</td> <td>%i</td> <td>%r:%rp</td>").contains("%t");
    messages = kconf->readBoolEntry("messages", true);
    messages_con = kconf->readBoolEntry("messages_connected", true);
    messages_discon = kconf->readBoolEntry("messages_disconnected", true);

    KRadioAction *r5;

    if((r5 = (KRadioAction *)actionCollection()->action(QString::fromLatin1("kconnections_type%1").arg(ign->ignoreType()))))
        r5->setChecked(true);
}

void KConnections::updateGeometries()
{
    QRect rc = KWin::windowInfo(winId()).geometry();

    label->setXY(rc.x(), rc.y());

    tip = new KWDynamicTip(this);
    tip->setText(i18n("No connections"));

    QToolTip::add(this, rc, i18n("No connections"));
}

void KConnections::slotAcceptAll()
{
    kconf->setGroup("General");
    kconf->writeEntry("ignore_type", 0);
    ign->rereadType();
}

void KConnections::slotIgnoreAll()
{
    kconf->setGroup("General");
    kconf->writeEntry("ignore_type", 1);
    ign->rereadType();
}

void KConnections::slotBlack()
{
    kconf->setGroup("General");
    kconf->writeEntry("ignore_type", 2);
    ign->rereadType();
}

void KConnections::slotWhite()
{
    kconf->setGroup("General");
    kconf->writeEntry("ignore_type", 3);
    ign->rereadType();
}

#include "kconnections.moc"
