/***************************************************************************
                          kwentry.cpp  -  description
                             -------------------
    begin                : ??? Dec 4 2007
    copyright            : (C) 2007 by Krasu
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qstringlist.h>
#include <qregexp.h>

#include <klocale.h>
#include <kdebug.h>

#include "kwentry.h"
#include "entry.h"
#include "kwignore.h"

#include <sys/types.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pwd.h>

KWEntry::KWEntry() : QObject(), QValueVector<Entry>()
{}

KWEntry::~KWEntry()
{}

void KWEntry::splitAndAdd(const QString &line)
{
    if(!line.length())
        return;

    QStringList tmp;
    QStringList list = QStringList::split(QRegExp("\\s"), line);

    if(list.count() < 8)
    {
        kdDebug() << "netstat output is broken, " << list.count() << " elements" << endl;
        return;
    }

    QStringList::iterator it = list.begin();
    Entry e;

    e.protocol = *it;
    ++it;

    e.req = (*it).toInt();
    ++it;

    e.sent = (*it).toInt();
    ++it;

    tmp = QStringList::split(QChar(':'), *it);
    e.addressLocal = tmp[0];
    e.addressLocalPort = tmp[1].toInt();
    ++it;

    tmp = QStringList::split(QChar(':'), *it);
    e.addressRemote = tmp[0];
    e.addressRemotePort = tmp[1].toInt();
    ++it;

    // hell! netstat's output is broken, in some
    // cases it will output smth like "ESTABLISHED1000",
    // not "ESTABLISHED 1000"
    e.state = *it;
    ++it;

    int idx;
    if((idx = e.state.find(QRegExp("[0-9]"))) != -1 &&
        !e.state.startsWith("FIN_WAIT1") &&
        !e.state.startsWith("FIN_WAIT2"))
    {
        QString t = e.state;

        e.state.truncate(idx);
        e.user = t.right(t.length()-idx).toInt();
    }
    else if(idx != -1 && e.state.length() > 9)
    {
        QString t = e.state;

        idx++;
        e.state.truncate(idx);
        e.user = t.right(t.length()-idx).toInt();
    }
    else
    {
        e.user = (*it).toInt();
        ++it;
    }

    e.inode = (*it).toInt();

    ++it;

    QString t = (*it);

    if((idx = t.find(QChar('/'))) > 0)
    {
        t = t.right(t.length()-idx-1);
        if(!t.startsWith(QChar('-')))
            e.app = t;
    }

    servent *se = getservbyport(htons(e.addressLocalPort), e.protocol.ascii());

    if(!se)
    {
        e.incomingPort = e.addressRemotePort;
        e.incoming = false;
        e.incomingString = ">";
        se = getservbyport(htons(e.addressRemotePort), e.protocol.ascii());
    }
    else
        e.incomingPort = e.addressLocalPort;

    e.service = se ? se->s_name : i18n("unknown");

    passwd *pwd = getpwuid(e.user);

    if(pwd)
        e.userString = pwd->pw_name;

    if(KWIgnore::instance()->search(e))
        return;

    if(!searchEntry(e))
    {
        e.mark = true;
        e.time = QDateTime::currentDateTime();
        append(e);
        emit added(e);
    }
}

bool KWEntry::searchEntry(const Entry &e)
{
    iterator itEnd = end();

    for(iterator it = begin();it != itEnd;++it)
    {
        if(*it == e)
        {
            (*it).mark = true;
            return true;
        }
    }

    return false;
}

void KWEntry::unmark()
{
    iterator itEnd = end();

    for(iterator it = begin();it != itEnd;++it)
    {
        (*it).mark = false;
    }
}

#include "kwentry.moc"
