/***************************************************************************
                          kwignore.cpp  -  description
                             -------------------
    begin                : ??? Dec 6 2007
    copyright            : (C) 2007 by Krasu
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kwignore.h"
#include "kwconfig.h"

#include <unistd.h>

KWIgnore * KWIgnore::m_inst = 0;

KWIgnore::KWIgnore()
{
    m_inst = this;

    reread();
}

KWIgnore::~KWIgnore()
{}

void KWIgnore::reread()
{
    KWConfig *konf = KWConfig::instance();

    rereadType();

    ignore.clear();
    konf->setGroup("Ignore");
    rereadPrivate(&ignore);

    accept.clear();
    konf->setGroup("Accept");
    rereadPrivate(&accept);
}

void KWIgnore::rereadType()
{
    KWConfig::instance()->setGroup("General");
    ignType = static_cast<Type>(KWConfig::instance()->readNumEntry("ignore_type", 3));
}

void KWIgnore::rereadPrivate(QValueVector<Entry> *listIgn)
{
    KWConfig *konf = KWConfig::instance();
    QStringList list;
    QStringList::iterator it, itEnd;
    int i = 0;
    QChar minus('-');

    while(true)
    {
    	list = konf->readListEntry(QString::number(i));

        if(list.count() < 3)
            break;

    	Entry e;

    	it = list.begin();
        itEnd = list.end();

        e.enabled = (*it).startsWith(QChar('1'));

        if(!e.enabled)
        {
            i++;
            continue;
        }

        ++it;

    	e.protocol = *it;
	++it;
        e.count += (!e.protocol.startsWith(minus));

    	e.service = *it;
	++it;
        e.count += (!e.service.startsWith(minus));

    	e.userString = *it;
	++it;
        e.count += (!e.userString.startsWith(minus));

    	if(it != itEnd)
        {
            e.app = *it;
            ++it;
            e.count += (!e.app.isEmpty());
        }

    	if(it != itEnd)
        {
	    e.addressLocal = *it;
    	    ++it;
            e.count += (!e.addressLocal.isEmpty());
	}

    	if(it != itEnd)
	{
	    e.addressLocalPort = (*it).toInt();
    	    ++it;
            e.count += (bool)(e.addressLocalPort);
	}

    	if(it != itEnd)
	{
	    e.addressRemote = *it;
            e.count += (!e.addressLocal.isEmpty());
    	    ++it;
	}

    	if(it != itEnd)
        {
    	    e.addressRemotePort = (*it).toInt();
            e.count += (bool)(e.addressRemotePort);
        }

        listIgn->append(e);

    	i++;
    }
}

bool KWIgnore::search(const Entry &e)
{
    // accept all connections?
    if(ignType == Accept)
        return false;
    // reject all connections?
    else if(ignType == Reject)
        return true;

    QValueVector<Entry> *ign = (ignType == Black ? &ignore : &accept);

    QValueVector<Entry>::const_iterator itEnd = ign->end();
    Entry ent;
    char cuser[L_cuserid+1];
    cuserid(cuser);
    int cnt;

    for(QValueVector<Entry>::const_iterator it = ign->begin();it != itEnd;++it)
    {
        ent = *it;
        cnt = 0;

        if(e.userString == ent.userString || (ent.userString == "$me" && e.userString == cuser))
            cnt++;

        if(e.protocol == ent.protocol)
            cnt++;

        if(e.service == ent.service)
            cnt++;

        if(e.app == ent.app)
            cnt++;

        if(e.addressLocal == ent.addressLocal)
            cnt++;

        if(e.addressLocalPort == ent.addressLocalPort)
            cnt++;

        if(e.addressRemote == ent.addressRemote)
            cnt++;

        if(e.addressRemotePort == ent.addressRemotePort)
            cnt++;

        // found something...
        if(cnt == ent.count)
        {
            // ignType == Black, ignore connection
            // ignType == White, accept connection
            return ignType == Black;
        }
    }

    // connection not found
    // if ignType == White, we should ignore this connection
    // if ignType == Black, we should accept this connection
    return ignType == White;
}
